/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.security.auth;

import com.liferay.portal.NoSuchRoleException;
import com.liferay.portal.NoSuchUserException;
import com.liferay.portal.NoSuchUserGroupException;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.LocaleUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.model.Role;
import com.liferay.portal.model.UserGroup;
import com.liferay.portal.model.User;
import com.liferay.portal.service.RoleLocalServiceUtil;
import com.liferay.portal.service.UserGroupLocalServiceUtil;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.servlet.filters.sso.opensso.OpenSSOUtil;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portal.util.PrefsPropsUtil;
import com.liferay.portal.util.PropsKeys;
import com.liferay.portal.util.PropsValues;
import com.liferay.util.PwdGenerator;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * <a href="OpenSSOAutoLogin.java.html"><b><i>View Source</i></b></a>
 *
 * @author Brian Wing Shun Chan
 * @author Prashant Dighe
 *
 */
public class OpenSSOAutoLogin implements AutoLogin {

	public String[] login(
		HttpServletRequest request, HttpServletResponse response) {

		String[] credentials = null;

		try {
			long companyId = PortalUtil.getCompanyId(request);

			if (!PrefsPropsUtil.getBoolean(
					companyId, PropsKeys.OPEN_SSO_AUTH_ENABLED,
					PropsValues.OPEN_SSO_AUTH_ENABLED)) {

				return credentials;
			}

			String serviceUrl = PrefsPropsUtil.getString(
				companyId, PropsKeys.OPEN_SSO_SERVICE_URL);

			if (!OpenSSOUtil.isAuthenticated(request, serviceUrl)) {
				return credentials;
			}

			String screenNameAttr = PrefsPropsUtil.getString(
				companyId, PropsKeys.OPEN_SSO_SCREEN_NAME_ATTR,
				PropsValues.OPEN_SSO_SCREEN_NAME_ATTR);
			String emailAddressAttr = PrefsPropsUtil.getString(
				companyId, PropsKeys.OPEN_SSO_EMAIL_ADDRESS_ATTR,
				PropsValues.OPEN_SSO_EMAIL_ADDRESS_ATTR);
			String firstNameAttr = PrefsPropsUtil.getString(
				companyId, PropsKeys.OPEN_SSO_FIRST_NAME_ATTR,
				PropsValues.OPEN_SSO_FIRST_NAME_ATTR);
			String lastNameAttr = PrefsPropsUtil.getString(
				companyId, PropsKeys.OPEN_SSO_LAST_NAME_ATTR,
				PropsValues.OPEN_SSO_LAST_NAME_ATTR);
			
			//read opensso settings
			boolean convert_role=PrefsPropsUtil.getBoolean(PropsKeys.OPEN_SSO_CONVERT_GROUP_TO_ROLE,
					PropsValues.OPEN_SSO_CONVERT_GROUP_TO_ROLE);
			boolean convert_group=PrefsPropsUtil.getBoolean(PropsKeys.OPEN_SSO_CONVERT_GROUP_TO_GROUP,
					PropsValues.OPEN_SSO_CONVERT_GROUP_TO_GROUP);
			boolean auto_role=PrefsPropsUtil.getBoolean(PropsKeys.OPEN_SSO_AUTO_CREATE_ROLE,
					PropsValues.OPEN_SSO_AUTO_CREATE_ROLE);
			boolean auto_group=PrefsPropsUtil.getBoolean(PropsKeys.OPEN_SSO_AUTO_CREATE_GROUP,
					PropsValues.OPEN_SSO_AUTO_CREATE_GROUP);
			

			Map<String, String> nameValues = OpenSSOUtil.getAttributes(
				request, serviceUrl);

			String screenName = nameValues.get(screenNameAttr);
			String emailAddress = nameValues.get(emailAddressAttr);
			String firstName = nameValues.get(firstNameAttr);
			String lastName = nameValues.get(lastNameAttr);

			if (Validator.isNull(emailAddress)) {
				throw new AutoLoginException("Email address is null");
			}

			User user = null;
			long UserId=0;
			

			try {
				user = UserLocalServiceUtil.getUserByScreenName(
					companyId, screenName);
				
//*** If user already exist into liferay database, if enabled, update user's Group and Role *********************
				
				if (convert_role){ 
						System.out.println("OpenSSO Convert Group To Role Enabled for update");
						updateRoles(nameValues, companyId, screenName, auto_role);}				
				else System.out.println("OpenSSO Convert Group To Role Disabled for update");

				if (convert_group){ 
						System.out.println("OpenSSO Convert Group To GROUP Enabled for update");
						updateGroups(nameValues, companyId, screenName, auto_group);}
				else System.out.println("OpenSSO Convert Group To Group Disabled for update");
//****************************************************************************************************

			}
			catch (NoSuchUserException nsue) {
				ThemeDisplay themeDisplay = (ThemeDisplay)request.getAttribute(
					WebKeys.THEME_DISPLAY);

				Locale locale = LocaleUtil.getDefault();

				if (themeDisplay != null) {

					// ThemeDisplay should never be null, but some users
					// complain of this error. Cause is unknown.

					locale = themeDisplay.getLocale();
				}

				user = addUser(companyId, firstName, lastName, emailAddress, screenName,locale);

//*** If enabled, set Roles and/or Groups for new User ***********************************************************
				
				//get userId from liferay database
				UserId=UserLocalServiceUtil.getUserIdByScreenName(companyId, screenName);

				//set user's role from opensso group to Liferay Role
				if (convert_role){ 
					System.out.println("OpenSSO Convert Group To Role Enabled for new");
					setRoles(nameValues, UserId, companyId, auto_role);}
				else System.out.println("OpenSSO Convert Group To Role Disabled for new");

				//set user's group from opensso group to Liferay Group
				if (convert_group){ 
					System.out.println("OpenSSO Convert Group To GROUP Enabled for new");
					setGroups(nameValues, UserId, companyId, auto_group);}
				else System.out.println("OpenSSO Convert Group To group Disabled for new");

//***************************************************************************************
			}

			credentials = new String[3];

			credentials[0] = String.valueOf(user.getUserId());
			credentials[1] = user.getPassword();
			credentials[2] = Boolean.TRUE.toString();
		}
		catch (Exception e) {
			_log.error(e, e);
		}

		return credentials;
	}

	protected User addUser(
			long companyId, String firstName, String lastName,
			String emailAddress, String screenName, Locale locale)
		throws Exception {

		long creatorUserId = 0;
		boolean autoPassword = false;
		String password1 = PwdGenerator.getPassword();
		String password2 = password1;
		boolean autoScreenName = false;
		String openId = StringPool.BLANK;
		String middleName = StringPool.BLANK;
		int prefixId = 0;
		int suffixId = 0;
		boolean male = true;
		int birthdayMonth = Calendar.JANUARY;
		int birthdayDay = 1;
		int birthdayYear = 1970;
		String jobTitle = StringPool.BLANK;
		long[] groupIds = null;
		long[] organizationIds = null;
		long[] roleIds = null;
		long[] userGroupIds = null;
		boolean sendEmail = false;
		ServiceContext serviceContext = new ServiceContext();

		return UserLocalServiceUtil.addUser(
			creatorUserId, companyId, autoPassword, password1, password2,
			autoScreenName, screenName, emailAddress, openId, locale, firstName,
			middleName, lastName, prefixId, suffixId, male, birthdayMonth,
			birthdayDay, birthdayYear, jobTitle, groupIds, organizationIds,
			roleIds, userGroupIds, sendEmail, serviceContext);
	}

	private static Log _log = LogFactoryUtil.getLog(OpenSSOAutoLogin.class);
	
//*********************get user's group name from opensso	************************************************
	protected List<String> getRoles(Map<String, String> nameValues){
		String name="memberof";
		List<String> Roles = new ArrayList<String>();
		int x=1;
		while(nameValues.get(name+x)!=null){ 
			Roles.add(nameValues.get(name+x));
			x++;
		}
		return Roles;
	}
//*******************************************************************************************************
	
	
//*********************set user's Role from opensso group to liferay role**********************************	
	protected void setRoles(Map<String, String> nameValues, long UserId, long companyId, boolean auto_role) throws PortalException, SystemException, NoSuchRoleException{
		
		List<String> UserRoles=getRoles(nameValues);
		Iterator<String> itr = UserRoles.iterator();
		long[] UserIds = new long[1];
		UserIds[0]=UserId;
		long RoleId=0;
		String RoleName=null;
			while(itr.hasNext()){ 
				RoleName=itr.next();
				
			try{	
				//get role ID from liferay
				RoleId=RoleLocalServiceUtil.getRole(companyId, RoleName).getRoleId();
				
				//set user's group to liferay user
				UserLocalServiceUtil.setRoleUsers(RoleId, UserIds);
				}
			catch (NoSuchRoleException e){ 
					if (auto_role){
						RoleLocalServiceUtil.addRole(UserId, companyId, RoleName, "Created Automatically from OpenSSO", 0);
						RoleId=RoleLocalServiceUtil.getRole(companyId, RoleName).getRoleId();
						UserLocalServiceUtil.setRoleUsers(RoleId, UserIds);
					}
				}
			}
	}
//**********************************************************************************************

	
	
//*** Update user role from opensso group to liferay role **************************************
	protected void updateRoles(Map<String, String> nameValues, long companyId, String screenName, boolean auto_role) throws PortalException, SystemException, NoSuchRoleException{
		
		//get userid from liferay
		Long UserId=UserLocalServiceUtil.getUserIdByScreenName(companyId, screenName); 
		
		//get user's roles from liferay
		List<Role> UserRoles = RoleLocalServiceUtil.getUserRoles(UserId);						
		Iterator<Role> itrUserRoles=UserRoles.iterator();

		//get group name from opensso
		List<String> openssoRole = getRoles(nameValues);								
		
		Role role=null;
		while(itrUserRoles.hasNext()){
			role=itrUserRoles.next();
			//if the role is removed from opensso, then remove the role from lifery user
			if(!openssoRole.contains(role.getName())) UserLocalServiceUtil.deleteRoleUser(role.getRoleId(), UserId);	
			//else remove the group from list
			else openssoRole.remove(role.getName());																	
			}
		
		Iterator<String> itrOpenssoRoles=openssoRole.iterator();
		long RoleId=0;
		long[] UserIds = new long[1];
		UserIds[0]=UserId;
		String RoleName=null;
		
		//add the new role to liferay user
		while(itrOpenssoRoles.hasNext()){				
			RoleName=itrOpenssoRoles.next();
			
			
		try{
			//get role ID from liferay
			RoleId=RoleLocalServiceUtil.getRole(companyId, RoleName).getRoleId();
			
			UserLocalServiceUtil.setRoleUsers(RoleId, UserIds);
		
		}
		catch (NoSuchRoleException e){ 
				if (auto_role){
					
							RoleLocalServiceUtil.addRole(UserIds[0], companyId, RoleName, "Created Automatically from OpenSSO", 0);
							RoleId=RoleLocalServiceUtil.getRole(companyId, RoleName).getRoleId();
							UserLocalServiceUtil.setRoleUsers(RoleId, UserIds);
				}}				
		}
	}
//********************************************************************************************************	

	
//SET_GROUPS***********************set user's group from opensso to liferay group**************************	
	protected void setGroups(Map<String, String> nameValues, long UserId, long companyId, boolean auto_group) throws PortalException, SystemException, NoSuchRoleException{
		
		List<String> UserGroups=getRoles(nameValues);
		Iterator<String> itr = UserGroups.iterator();
		long[] UserIds = new long[1];
		UserIds[0]=UserId;
		long GroupId=0;
		String GroupName=null;
			while(itr.hasNext()){ 
				GroupName=itr.next();
				
			try{	
				//get role ID from liferay
				GroupId=RoleLocalServiceUtil.getRole(companyId, GroupName).getRoleId();
				
				//set user's group to liferay user
				UserLocalServiceUtil.setUserGroupUsers(GroupId, UserIds);}
				
			catch (NoSuchRoleException e){ 
					if (auto_group){
												
						UserGroupLocalServiceUtil.addUserGroup(UserId, companyId, GroupName, "Created Automatically from OpenSSO");
						GroupId=RoleLocalServiceUtil.getRole(companyId, GroupName).getRoleId();
						
						//set User's Group
						UserLocalServiceUtil.setUserGroupUsers(GroupId, UserIds);
					}		
				}
			}
      }
	
//UPDATE_GROUP*****************************update user's group from opensso group to liferay group************
	
	protected void updateGroups(Map<String, String> nameValues, long companyId, String screenName, boolean auto_group) throws PortalException, SystemException, NoSuchRoleException{
		
		//get userid from liferay
		Long UserId=UserLocalServiceUtil.getUserIdByScreenName(companyId, screenName); 
		long[] UserIds = new long[1];
		UserIds[0]=UserId;
		//get user's roles from liferay
		List<UserGroup> UserGroups = UserGroupLocalServiceUtil.getUserUserGroups(UserId);						
		Iterator<UserGroup> itrUserGroups=UserGroups.iterator();

		//get group name from opensso
		List<String> openssoRole = getRoles(nameValues);								
		
		UserGroup Group=null;
		while(itrUserGroups.hasNext()){
			Group=itrUserGroups.next();
			//if the role is removed from opensso, then remove the role from lifery user
			if(!openssoRole.contains(Group.getName())) UserLocalServiceUtil.unsetUserGroupUsers(Group.getUserGroupId(), UserIds);	
			//else remove the group from list
			else openssoRole.remove(Group.getName());																	
			}
		
		Iterator<String> itrOpenssoRoles=openssoRole.iterator();
		long GroupId=0;
		
		
		String GroupName=null;
		
		//add the new role to liferay user
		while(itrOpenssoRoles.hasNext()){				
			GroupName=itrOpenssoRoles.next();
			
			
		try{
			//get role ID from liferay
			GroupId=UserGroupLocalServiceUtil.getUserGroup(companyId, GroupName).getUserGroupId();
			
			UserLocalServiceUtil.setUserGroupUsers(GroupId, UserIds);		
		}
		catch (NoSuchUserGroupException e){ 
				if (auto_group){
					
							UserGroupLocalServiceUtil.addUserGroup(UserId, companyId, GroupName, "Created Automatically from OpenSSO");
							GroupId=UserGroupLocalServiceUtil.getUserGroup(companyId, GroupName).getUserGroupId();
							
							UserLocalServiceUtil.setUserGroupUsers(GroupId, UserIds);
				}}			
		}
	}
}

